#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <straight/straight.h>

char inputFileName[256] = "";
char outputFileName[256] = "";

char outputPluginName[256] = "output_wav";

int main(int argc, char *argv[])
{
    StraightConfig config;
    Straight straight;
    StraightSource source;
    StraightSpecgram specgram;
    StraightSynth synth;
    StraightFile sf;
    char chunkId[8];
    long chunkSize;

    if (argc < 3) {
        fprintf(stderr, "readstf <input STF file> <output file>\n");
        exit(1);
    }
    
    strcpy(inputFileName, argv[1]);
    strcpy(outputFileName, argv[2]);

#ifdef USE_TANDEM
    straightInitConfigTandem(&config);
#else
    straightInitConfig(&config);
#endif
    config.samplingFrequency = 16000.0;

    if ((straight = straightInitialize(&config)) != NULL) {
        if ((sf = straightFileOpen(inputFileName, "r")) != NULL) {
            fprintf(stderr, "Input file: %s\n", inputFileName);
        
            if (straightFileLoadHeader(sf, straight) == ST_TRUE) {
                source = straightSourceInitialize(straight, NULL);
                specgram = straightSpecgramInitialize(straight, NULL);
            
                while (straightFileLoadChunkInfo(sf, chunkId, &chunkSize) == ST_TRUE) {
#ifdef DEBUG
                    fprintf(stderr, "DEBUG: chunkId = %s, chunkSize = %ld\n", chunkId, chunkSize);
#endif
                    if (strcmp(chunkId, "F0  ") == 0) {
                        straightFileLoadF0(sf, source);
                    } else if (strcmp(chunkId, "UVF0") == 0) {
                        straightFileLoadUnvoicedF0(sf, source);
                    } else if (strcmp(chunkId, "PRLV") == 0) {
                        straightFileLoadPeriodicityLevel(sf, source);
                    } else if (strcmp(chunkId, "F0CN") == 0) {
                        straightFileLoadF0Candidates(sf, source);
                    } else if (strcmp(chunkId, "AP  ") == 0 || strcmp(chunkId, "APSG") == 0) {
                        straightFileLoadAperiodicity(sf, source);
                    } else if (strcmp(chunkId, "SPEC") == 0) {
                        straightFileLoadSpecgram(sf, specgram);
                    } else {
                        straightFileSkipChunk(sf, chunkSize);
                    }
                }

                synth = straightSynthInitialize(straight, NULL);
                if (straightSynthCompute(straight, source, specgram, synth, 1.0, 1.0, 1.0) == ST_TRUE) {
                    straightLimitSynthWaveAmplitude(straight, synth, 0.9765625);
                    
                    if (straightWriteSynthAudioFile(straight, synth,
                                                    outputFileName, outputPluginName, 16) == ST_TRUE) {
                        fprintf(stderr, "Output file: %s\n", outputFileName);
                    } else {
                        fprintf(stderr, "Error: Cannot open file: %s\n", outputFileName);
                    }
                }
    
                straightSourceDestroy(source);
                straightSpecgramDestroy(specgram);
            } else {
                fprintf(stderr, "Error: Cannot load STF header: %s\n", inputFileName);
            }
            
            straightFileClose(sf);
        } else {
            fprintf(stderr, "Error: Cannot open file: %s\n", inputFileName);
        }
            
        straightDestroy(straight);
    } else {
        fprintf(stderr, "Error: Cannot initialize STRAIGHT engine\n");
    }

    return 0;
}
