#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <straight/straight.h>

char inputFileName[256] = "";
char outputFileName[256] = "";

char outputPluginName[256] = "output_wav";

static stBool asyncCallbackFunc(StraightAsync async, stCallbackType callbackType,
                                void *callbackData, void *userData)
{
    int percent;
    StraightSource source;
    StraightSpecgram specgram;
    StraightSynth synth;
    
    /*fprintf(stderr, "Callback: callbackType = %lx\n", callbackType);*/
    
    if (callbackType == STRAIGHT_F0_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT F0: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_AP_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT AP: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_SPECGRAM_PERCENTAGE_CALLBACK) {
        percent = (int)callbackData;
        fprintf(stderr, "STRAIGHT spectrogram: %d %%\r", percent);
        if (percent >= 100) {
            fprintf(stderr, "\n");
        }
    } else if (callbackType == STRAIGHT_F0_COMPUTED_CALLBACK) {
        source = (StraightSource)callbackData;
        
        fprintf(stderr, "STRAIGHT source: F0 computed\n");
        
        straightAsyncSetEvent(async);
    } else if (callbackType == STRAIGHT_AP_COMPUTED_CALLBACK) {
        source = (StraightSource)callbackData;
        
        fprintf(stderr, "STRAIGHT source: AP computed\n");
        
        straightAsyncSetEvent(async);
    } else if (callbackType == STRAIGHT_SPECGRAM_COMPUTED_CALLBACK) {
        specgram = (StraightSpecgram)callbackData;
        
        fprintf(stderr, "STRAIGHT specgram: computed\n");
        
        straightAsyncSetEvent(async);
    } else if (callbackType == STRAIGHT_SYNTH_COMPUTED_CALLBACK) {
        synth = (StraightSynth)callbackData;
        
        fprintf(stderr, "STRAIGHT synth: computed\n");
        
        straightLimitSynthWaveAmplitude(straightAsyncGetStraight(async), synth, 0.9765625);
        
        if (straightWriteSynthAudioFile(straightAsyncGetStraight(async), synth,
                                        outputFileName, outputPluginName, 16) == ST_TRUE) {
            fprintf(stderr, "Output file: %s\n", outputFileName);
        } else {
            fprintf(stderr, "Error: Cannot open file: %s\n", outputFileName);
        } 
        
        straightAsyncSetEvent(async);
    }

    return ST_TRUE;
}

int main(int argc, char *argv[])
{
    int i;
    StraightConfig config;
    Straight straight;
    StraightAsync async;
    StraightSource source;
    StraightSpecgram specgram;
    StraightSynth synth;
    stBool computeSpecgramFlag = ST_FALSE;

    if (argc < 3) {
        fprintf(stderr, "simple <input file> <output file>\n");
        exit(1);
    }
    
    strcpy(inputFileName, argv[1]);
    strcpy(outputFileName, argv[2]);

#ifdef USE_TANDEM
    straightInitConfigTandem(&config);
#else
    straightInitConfig(&config);
#endif
    config.samplingFrequency = 16000.0;

    if ((straight = straightInitialize(&config)) != NULL) {
        if (straightReadAudioFile(straight, inputFileName, NULL) == ST_FALSE) {
            fprintf(stderr, "Error: Cannot open file: %s\n", inputFileName);
        } else {
            fprintf(stderr, "Input file: %s\n", inputFileName);
            
            async = straightAsyncInitialize(straight,
                                            STRAIGHT_PERCENTAGE_CALLBACK | STRAIGHT_COMPUTED_CALLBACK,
                                            asyncCallbackFunc, NULL);
            
            if (async != NULL) {
                source = straightSourceInitialize(straight, NULL);
                specgram = straightSpecgramInitialize(straight, NULL);
                synth = straightSynthInitialize(straight, NULL);

                straightAsyncResetEvent(async);
                
                for (i = 0;; i++) {
                    if (straightSynthGetState(synth) >= STRAIGHT_STATE_SYNTH_COMPUTED) {
                        break;
                    }
                    
                    if (i == 0) {
                        straightSourceComputeF0Async(async, source);
                    } else if (computeSpecgramFlag == ST_FALSE) {
                        if (straightSourceGetState(source) >= STRAIGHT_STATE_F0_COMPUTED) {
                            straightSourceComputeAperiodicityAsync(async, source);
                            straightSpecgramComputeAsync(async, source, specgram);
                            computeSpecgramFlag = ST_TRUE;
                        }
                    } else if (straightSourceGetState(source) >= STRAIGHT_STATE_AP_COMPUTED
                               && straightSpecgramGetState(specgram) >= STRAIGHT_STATE_SPECGRAM_COMPUTED) {
                        straightSynthComputeAsync(async, source, specgram, synth,
                                                  1.0, 1.0, 1.0);
                    }

                    /*fprintf(stderr, "async waiting...\n");*/
                    straightAsyncWaitEvent(async);
                    /*fprintf(stderr, "async waiting done\n");*/
                    straightAsyncResetEvent(async);
                }
    
                straightSourceDestroy(source);
                straightSpecgramDestroy(specgram);
                straightSynthDestroy(synth);

                straightAsyncDestroy(async);
            }
        }
        straightDestroy(straight);
    } else {
        fprintf(stderr, "Error: Cannot initialize STRAIGHT engine\n");
    }

    return 0;
}
