#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <straight/straight.h>

char inputFileName[256] = "";
char outputFileName[256] = "";

int main(int argc, char *argv[])
{
    StraightConfig config;
    Straight straight;
    StraightSource source;
    StraightSpecgram specgram;
    StraightFile sf;
    char chunkList[128];

    if (argc < 3) {
        fprintf(stderr, "writestf <input file> <output STF file>\n");
        exit(1);
    }
    
    strcpy(inputFileName, argv[1]);
    strcpy(outputFileName, argv[2]);

#ifdef USE_TANDEM
    straightInitConfigTandem(&config);
#else
    straightInitConfig(&config);
    strcpy(chunkList, "F0  AP  SPEC");
#endif
    config.samplingFrequency = 16000.0;

    if ((straight = straightInitialize(&config)) != NULL) {
        if (straightReadAudioFile(straight, inputFileName, NULL) == ST_FALSE) {
            fprintf(stderr, "Error: Cannot open file: %s\n", inputFileName);
        } else {
            fprintf(stderr, "Input file: %s\n", inputFileName);
            
            source = straightSourceInitialize(straight, NULL);
            straightSourceCompute(straight, source, NULL, 0);

            specgram = straightSpecgramInitialize(straight, NULL);
            straightSpecgramCompute(straight, source, specgram, NULL, 0);

            if ((sf = straightFileOpen(outputFileName, "w")) != NULL) {
                fprintf(stderr, "Output file: %s\n", outputFileName);

#ifdef USE_TANDEM
                strcpy(chunkList, "F0  ");
                if (straightSourceIsUnvoicedF0Supported(source)) {
                    strcat(chunkList, "UVF0PRLV");
                }
                if (straightSourceIsF0CandidatesSupported(source)) {
                    strcat(chunkList, "F0CN");
                }
                strcat(chunkList, "AP  SPEC");
#endif
                straightFileSetChunkList(sf, chunkList);
                
                straightFileWriteHeader(sf, straight);
                straightFileWriteF0Chunk(sf, source);
#ifdef USE_TANDEM
                if (straightSourceIsUnvoicedF0Supported(source)) {
                    straightFileWriteUnvoicedF0Chunk(sf, source);
                    straightFileWritePeriodicityLevelChunk(sf, source);
                }
                if (straightSourceIsF0CandidatesSupported(source)) {
                    straightFileWriteF0CandidatesChunk(sf, source);
                }
#endif
                straightFileWriteAperiodicityChunk(sf, source);
                straightFileWriteSpecgramChunk(sf, specgram);

                straightFileClose(sf);
            } else {
                fprintf(stderr, "Error: Cannot open file: %s\n", outputFileName);
            }
                
            straightSourceDestroy(source);
            straightSpecgramDestroy(specgram);
        }
        straightDestroy(straight);
    } else {
        fprintf(stderr, "Error: Cannot initialize STRAIGHT engine\n");
    }

    return 0;
}
