/*
 *      straightSource.h
 */

#ifndef __STRAIGHTSOURCE_H
#define __STRAIGHTSOURCE_H

#include <straight/straightDefs.h>

#ifdef __cplusplus
extern "C" {
#endif
    
typedef struct _StraightSourceConfigGeneric {
    unsigned long versionId;
    unsigned long engineVersionId;
} StraightSourceConfigGeneric;
    
typedef struct _StraightSourceConfigV40 {
    unsigned long versionId;
    unsigned long engineVersionId;

    long numChannelsInOctave;	/* nvo */
    
    double windowStretchFactor;	/* mu */
    double smoothingLengthRelToFc; /* smp */
    double minimumSmoothingLength; /* minsm */
    double exponentForNonlinearSum; /* pcf0 */

    long numHarmonicsForInitialEstimate; /* nh */
} StraightSourceConfigV40;

#define StraightSourceConfig StraightSourceConfigV40

typedef struct _StraightSourceConfigTandemV008 {
    unsigned long versionId;
    unsigned long engineVersionId;

    long channelsPerOctave;
    long numberOfHarmonicsForExtraction;
    long numberOfHarmonicsForRefinement;

    double vuvThreshold;
} StraightSourceConfigTandemV008;

typedef struct _StraightSourceConfigTandemV009 {
    unsigned long versionId;
    unsigned long engineVersionId;

    long channelsPerOctave;
    long numberOfHarmonicsForExtraction;
    long numberOfHarmonicsForRefinement;

    double vuvThreshold;

    /*---- version 1.1.0 ----*/
    double periodicityShaping;
    long numberOfF0Candidates;

    double aperiodicityExponent;
    long aperiodicitySideMargin;
    stBool sigmoidAperiodicity;
} StraightSourceConfigTandemV009;

#define StraightSourceConfigTandem StraightSourceConfigTandemV009
#define _StraightSourceConfigTandem _StraightSourceConfigTandemV009
    
#if defined(MACOS)
#pragma import on
#endif

extern stBool _straightSourceInitConfig(StraightSourceConfig *config, unsigned long versionId, unsigned long engineVersionId);
#define straightSourceInitConfig(conf) _straightSourceInitConfig((StraightSourceConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_ENGINE_VERSION_ID)
#define straightSourceInitConfigV30(conf) _straightSourceInitConfig((StraightSourceConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_V30_ENGINE_VERSION_ID)
#define straightSourceInitConfigTandemV008(conf) _straightSourceInitConfig((StraightSourceConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_TANDEM_V008_ENGINE_VERSION_ID)
#define straightSourceInitConfigTandem(conf) _straightSourceInitConfig((StraightSourceConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_TANDEM_ENGINE_VERSION_ID)
#define straightSourceInitConfigWithEngine(conf,engineVersionId) _straightSourceInitConfig((StraightSourceConfig *)conf, STRAIGHT_VERSION_ID, engineVersionId) /* 1.1.0+ */

extern StraightSource _straightSourceInitialize(Straight straight, StraightSourceConfig *config);
#define straightSourceInitialize(str, conf) _straightSourceInitialize(str, (StraightSourceConfig *)conf)
extern stBool _straightSourceReset(StraightSource source, StraightSourceConfig *config); /* 1.1.0+ */
#define straightSourceReset(source, conf) _straightSourceReset(source, (StraightSourceConfig *)conf)
extern stBool straightSourceDestroy(StraightSource source);
extern stBool straightSourceClear(StraightSource source); /* 1.1.0+ */
    
extern stState straightSourceGetState(StraightSource source);
    
extern stBool straightSourceCompute(Straight straight, StraightSource source,
				    double *inputWave, long inputWaveLength);

extern double straightSourceQuantizePosition(Straight straight, StraightSource source,
					     double positionSec, long *frame);
extern double straightSourceGetFramePosition(Straight straight, StraightSource source, long frame);
    
extern long straightSourceGetF0NumFrames(StraightSource source);
extern stBool straightSourceGetF0(Straight straight, StraightSource source, long frame, double *value);
extern stBool straightSourceSetF0(Straight straight, StraightSource source, long frame, double value);
    
extern long straightSourceGetAperiodicityNumFrames(StraightSource source);
extern long straightSourceGetAperiodicityFrequencyLength(StraightSource source);
extern stBool straightSourceGetAperiodicity(Straight straight, StraightSource source, long frame, double *aperiodicityBuffer, long aperiodicityBufferLength);
extern stBool straightSourceSetAperiodicity(Straight straight, StraightSource source, long frame, double *aperiodicityBuffer, long aperiodicityBufferLength);
extern double *straightSourceGetAperiodicityPointer(Straight straight, StraightSource source, long frame);
    
extern stBool straightSourceCreate(Straight straight, StraightSource source, long nframes); /* F0 and AP */
extern stBool straightSourceCreateF0(Straight straight, StraightSource source, long nframes);
extern stBool straightSourceCreateAperiodicity(Straight straight, StraightSource source, long nframes);

extern stBool straightSourceIsFixedFrameShift(StraightSource source);

extern stBool straightSourceComputeF0Async(StraightAsync async, StraightSource source);
extern stBool straightSourceComputeAperiodicityAsync(StraightAsync async, StraightSource source);

/*---- version 1.1.0 ----*/
extern stBool straightSourceIsF0CandidatesSupported(StraightSource source);
extern stBool straightSourceCreateF0Candidates(Straight straight, StraightSource source, long ncandidates);
extern stBool straightSourceGetF0Candidate(Straight straight, StraightSource source, long frame, long candidate, double *value);
extern stBool straightSourceSetF0Candidate(Straight straight, StraightSource source, long frame, long candidate, double value);

extern stBool straightSourceIsPeriodicityLevelSupported(StraightSource source);
extern stBool straightSourceIsUnvoicedF0Supported(StraightSource source);
extern stBool straightSourceSetUnvoiced(Straight straight, StraightSource source, long frame);
extern stBool straightSourceGetUnvoicedF0(Straight straight, StraightSource source, long frame, double *value);
extern stBool straightSourceSetUnvoicedF0(Straight straight, StraightSource source, long frame, double value);
    
/*---- version 1.1.1 ----*/
extern double straightSourceGetF0ComputeTime(StraightSource source);
extern double straightSourceGetAperiodicityComputeTime(StraightSource source);

/*---- version 1.1.3 ----*/
extern long straightSourceGetTotalChannelsInF0Compute(StraightSource source);

#if defined(MACOS)
#pragma import off
#endif
    
#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __STRAIGHTSOURCE_H */
