/*
 *      straightSpecgram.h
 */

#ifndef __STRAIGHTSPECGRAM_H
#define __STRAIGHTSPECGRAM_H

#include <straight/straightDefs.h>

#ifdef __cplusplus
extern "C" {
#endif
    
typedef struct _StraightSpecgramConfigGeneric {
    unsigned long versionId;
    unsigned long engineVersionId;
} StraightSpecgramConfigGeneric;
    
typedef struct _StraightSpecgramConfigV40 {
    unsigned long versionId;
    unsigned long engineVersionId;

    stBool fastFlag;
    double timeWindowStretch; /* eta */
    double exponentForNonlinearity; /* pc */
    double timeDomainCompensation; /* mag */
} StraightSpecgramConfigV40;

#define StraightSpecgramConfig StraightSpecgramConfigV40
#define _StraightSpecgramConfig _StraightSpecgramConfigV40

typedef struct _StraightSpecgramConfigTandemV008 {
    unsigned long versionId;
    unsigned long engineVersionId;

    double compensationCoefficient;
    double correctionForBlackman;
    stBool dcRemoval;
} StraightSpecgramConfigTandemV008;

typedef struct _StraightSpecgramConfigTandemMP002 {
    unsigned long versionId;
    unsigned long engineVersionId;

    double compensationCoefficient;
    double correctionForBlackman;
    stBool dcRemoval;

    /*---- version 1.1.0 ----*/
    double exponentControl;
    stBool akagiriSmoothing;
    double akagiriSmoothingCoefficient;
} StraightSpecgramConfigTandemMP002;

#define StraightSpecgramConfigTandem StraightSpecgramConfigTandemMP002
#define _StraightSpecgramConfigTandem _StraightSpecgramConfigTandemMP002
    
#if defined(MACOS)
#pragma import on
#endif

extern stBool _straightSpecgramInitConfig(StraightSpecgramConfig *config, unsigned long versionId, unsigned long engineVersionId);
#define straightSpecgramInitConfig(conf) _straightSpecgramInitConfig((StraightSpecgramConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_ENGINE_VERSION_ID)
#define straightSpecgramInitConfigV30(conf) _straightSpecgramInitConfig((StraightSpecgramConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_V30_ENGINE_VERSION_ID)
#define straightSpecgramInitConfigTandemV008(conf) _straightSpecgramInitConfig((StraightSpecgramConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_TANDEM_V008_ENGINE_VERSION_ID)
#define straightSpecgramInitConfigTandem(conf) _straightSpecgramInitConfig((StraightSpecgramConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_TANDEM_ENGINE_VERSION_ID)
#define straightSpecgramInitConfigWithEngine(conf,engineVersionId) _straightSpecgramInitConfig((StraightSpecgramConfig *)conf, STRAIGHT_VERSION_ID, engineVersionId) /* 1.1.0+ */

extern StraightSpecgram _straightSpecgramInitialize(Straight straight, StraightSpecgramConfig *config);
#define straightSpecgramInitialize(str, conf) _straightSpecgramInitialize(str, (StraightSpecgramConfig *)conf)
extern stBool _straightSpecgramReset(StraightSpecgram specgram, StraightSpecgramConfig *config); /* 1.1.0+ */
#define straightSpecgramReset(specgram, conf) _straightSpecgramReset(specgram, (StraightSpecgramConfig *)conf)
extern stBool straightSpecgramClear(StraightSpecgram specgram); /* 1.1.0+ */
extern stBool straightSpecgramDestroy(StraightSpecgram specgram);

extern stState straightSpecgramGetState(StraightSpecgram specgram);
    
extern stBool straightSpecgramCompute(Straight straight, StraightSource source, StraightSpecgram specgram,
				      double *inputWave, long inputWaveLength);

extern long straightSpecgramGetNumFrames(StraightSpecgram specgram);
extern long straightSpecgramGetFrequencyLength(StraightSpecgram specgram);

extern double straightSpecgramQuantizePosition(Straight straight, StraightSpecgram specgram,
					       double positionSec, long *frame);
extern double straightSpecgramGetFramePosition(Straight straight, StraightSpecgram specgram, long frame);
    
extern stBool straightSpecgramGetSpectrum(Straight straight, StraightSpecgram specgram, long frame, double *specBuffer, long specBufferLength);
extern stBool straightSpecgramSetSpectrum(Straight straight, StraightSpecgram specgram, long frame, double *specBuffer, long specBufferLength);
extern double *straightSpecgramGetSpectrumPointer(Straight straight, StraightSpecgram specgram, long frame);

extern stBool straightSpecgramCreate(Straight straight, StraightSpecgram specgram, long nframes);

extern stBool straightSpecgramIsFixedFrameShift(StraightSpecgram specgram);
    
extern stBool straightSpecgramComputeAsync(StraightAsync async, StraightSource source, StraightSpecgram specgram);
    
/*---- version 1.1.1 ----*/
extern double straightSpecgramGetComputeTime(StraightSpecgram specgram);
    
#if defined(MACOS)
#pragma import off
#endif
    
#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __STRAIGHTSPECGRAM_H */
