/*
 *      straightSynth.h
 */

#ifndef __STRAIGHTSYNTH_H
#define __STRAIGHTSYNTH_H

#include <straight/straightDefs.h>

#ifdef __cplusplus
extern "C" {
#endif
    
typedef struct _StraightSynthConfigGeneric {
    unsigned long versionId;
    unsigned long engineVersionId;
} StraightSynthConfigGeneric;
    
typedef struct _StraightSynthConfigV40 {
    unsigned long versionId;
    unsigned long engineVersionId;

    double lowestF0;
    
    stBool zeroPhaseFlag;

    stBool groupDelayRandomizeFlag;
    double groupDelayStandardDeviation;	/* delsp */
    double groupDelaySpatialBandWidth; /* gdbw */
    double groupDelayRandomizeCornerFrequency; /* cornf */

    stBool groupDelayRatioToT0Flag; /* delfracind */
    double groupDelayRatioToT0; /* delfrac */
} StraightSynthConfigV40;

#define StraightSynthConfig StraightSynthConfigV40
#define _StraightSynthConfig _StraightSynthConfigV40
    
typedef struct _StraightSynthConfigTandemV009 {
    unsigned long versionId;
    unsigned long engineVersionId;

    double unvoicedF0;
    
    stBool zeroPhaseFlag;

    stBool groupDelayRandomizeFlag;
    double groupDelayStandardDeviation;	/* delsp */
    double groupDelaySpatialBandWidth; /* gdbw */
    double groupDelayRandomizeCornerFrequency; /* cornf */

    stBool groupDelayRatioToT0Flag; /* delfracind */
    double groupDelayRatioToT0; /* delfrac */
} StraightSynthConfigTandemV009;

#define StraightSynthConfigTandem StraightSynthConfigTandemV009
#define _StraightSynthConfigTandem _StraightSynthConfigTandemV009
    
#if defined(MACOS)
#pragma import on
#endif

extern stBool _straightSynthInitConfig(StraightSynthConfig *config, unsigned long versionId, unsigned long engineVersionId);
#define straightSynthInitConfig(conf) _straightSynthInitConfig((StraightSynthConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_ENGINE_VERSION_ID)
#define straightSynthInitConfigV30(conf) _straightSynthInitConfig((StraightSynthConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_V30_ENGINE_VERSION_ID)
#define straightSynthInitConfigTandem(conf) _straightSynthInitConfig((StraightSynthConfig *)conf, STRAIGHT_VERSION_ID, STRAIGHT_TANDEM_ENGINE_VERSION_ID)

extern StraightSynth _straightSynthInitialize(Straight straight, StraightSynthConfig *config);
#define straightSynthInitialize(str, conf) _straightSynthInitialize(str, (StraightSynthConfig *)conf)
    
extern stBool straightSynthDestroy(StraightSynth synth);

extern stState straightSynthGetState(StraightSynth synth);
    
extern stBool straightSynthCompute(Straight straight, StraightSource source, StraightSpecgram specgram, StraightSynth synth,
				   double pitchConversionRate, double frequencyAxisConversionRate, double timeAxisConversionRate);
extern double *straightSynthGetOutputWave(StraightSynth synth, long *outputLength);

extern stBool straightSynthComputeAsync(StraightAsync async, StraightSource source,
					StraightSpecgram specgram, StraightSynth synth,
					double pitchConversionRate, double frequencyAxisConversionRate, double timeAxisConversionRate);
    
/*---- version 1.1.1 ----*/
extern double straightSynthGetComputeTime(StraightSynth synth);
    
#if defined(MACOS)
#pragma import off
#endif
    
#ifdef __cplusplus
}  /* Close scope of 'extern "C"' declaration */
#endif

#endif /* __STRAIGHTSYNTH_H */
